--
-- manualUnloading Specialization
--
-- @author  Rival
-- contains parts of manualPipe.lua by Miichi77
-- @date  12/07/16
--
-- @version 1.1 @13/07/16
			-- fixed MP bug
-- @version 1.2 @14/08/16
			-- fixed helper bug (needed to enter combine to start unloading)

manualUnloading = {};

function manualUnloading.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Overloading, specializations);
end;

function manualUnloading:load(xmlFile)
	self.setManualUnloading = SpecializationUtil.callSpecializationsFunction("setManualUnloading");
	self.doManualUnloading = false;
	self.pipeRaycastDistance = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.pipe#raycastDistance"), 6);
	self.pipeRaycastDistanceOrig = self.pipeRaycastDistance;
end;
function manualUnloading:postLoad()
	self:setManualUnloading(self.doManualUnloading, true);
end;
function manualUnloading:delete()
end;
function manualUnloading:mouseEvent(posX, posY, isDown, isUp, button)
end;
function manualUnloading:keyEvent(unicode, sym, modifier, isDown)
end;
function manualUnloading:readStream(streamId, connection)
	 local doManualUnloading = streamReadBool(streamId);
	 self:setManualUnloading(doManualUnloading, true);
end;
function manualUnloading:writeStream(streamId, connection)   
	streamWriteBool(streamId, self.doManualUnloading);
end;
function manualUnloading:updateTick(dt)
end;
function manualUnloading:update(dt)
	--if self.isEntered then
		if not self.isAIThreshing and self.isEntered then
			if self.currentPipeState == 2 and self:getIsPipeUnloadingAllowed() and self.fillLevel > 0 then
				if self:getIsActiveForInput(false) and InputBinding.hasEvent(InputBinding.CD_ACTIVATE_UNLOADING) then
					self:setManualUnloading(not self.doManualUnloading, nil);
				end;
			else
				if self.doManualUnloading then
					self:setManualUnloading(false, nil);
				end;
			end;
		elseif self.isAIThreshing and not self.doManualUnloading then
			self:setManualUnloading(true, nil);
		end;
		if not self.isMotorStarted and self.doManualUnloading then
			self:setManualUnloading(false, nil);
		end;
		if self:findTrailerToUnload(self.currentFillType) == nil and self.doManualUnloading then
			self:setManualUnloading(false, nil);
		end;
	--end;
end;
function manualUnloading:setManualUnloading(state, noEventSend)
	self.doManualUnloading = state;
	if self.doManualUnloading then
		self.pipeRaycastDistance = self.pipeRaycastDistanceOrig; 
	else
		self.pipeRaycastDistance = 0.1;
	end;
	manualUnloadingEvent.sendEvent(self, self.doManualUnloading, noEventSend);
end;
function manualUnloading:draw()
	if self.currentPipeState == 2 and self:getIsPipeUnloadingAllowed() and self.fillLevel > 0 and not self.isAIThreshing then
		if self.doManualUnloading then
			g_currentMission:addHelpButtonText(g_i18n:getText("CD_STOP_UNLOADING"), InputBinding.CD_ACTIVATE_UNLOADING);
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("CD_START_UNLOADING"), InputBinding.CD_ACTIVATE_UNLOADING);
		end;
	end;
end;


-- manualUnloadingEvent

manualUnloadingEvent = {};
manualUnloadingEvent_mt = Class(manualUnloadingEvent, Event);

InitEventClass(manualUnloadingEvent, "manualUnloadingEvent");

function manualUnloadingEvent:emptyNew()  
	local self = Event:new(manualUnloadingEvent_mt );
	self.className="manualUnloadingEvent";
	return self;
end;

function manualUnloadingEvent:new(vehicle, doManualUnloading, streamId)
	local self = manualUnloadingEvent:emptyNew()
	self.vehicle = vehicle;
	self.doManualUnloading = doManualUnloading;
	return self;
end;

function manualUnloadingEvent:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	self.vehicle = networkGetObject(id);
	self.doManualUnloading = streamReadBool(streamId);
	self:run(connection);
end;

function manualUnloadingEvent:writeStream(streamId, connection)
	streamWriteInt32(streamId, networkGetObjectId(self.vehicle));	
	streamWriteBool(streamId, self.doManualUnloading);
end;

function manualUnloadingEvent:run(connection)
	if self.vehicle ~= nil then
		self.vehicle:setManualUnloading(self.doManualUnloading, true);
		if not connection:getIsServer() then
			g_server:broadcastEvent(manualUnloadingEvent:new(self.vehicle, self.doManualUnloading), nil, connection, self.vehicle);
		end;
	end;
end;

function manualUnloadingEvent.sendEvent(vehicle, doManualUnloading, noEventSend)  
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then   
			g_server:broadcastEvent(manualUnloadingEvent:new(vehicle, doManualUnloading), nil, nil, vehicle);
		else 
			g_client:getServerConnection():sendEvent(manualUnloadingEvent:new(vehicle, doManualUnloading));
		end;
	end;
end;